@@ -70,7 +70,6 @@ import psutil  # type: ignore
 
 from .mperror import MPError
 from . import streamer
-from . import async_streamer
 
 
 logger = logging.getLogger(__name__)
@@ -192,13 +191,14 @@ class Servlet(metaclass=ABCMeta):
                     try:
                         # If there is remaining time, wait up to
                         # that long.
-                        # Otherwise, get the next item it is there
+                        # Otherwise, get the next item if it is there
                         # right now (i.e. no waiting) even if we
                         # are already over time. That is, if supply
                         # has piled up, then will get up to the
                         # batch capacity.
                         if time_left > 0:
-                            uid, x = q_in.get(timeout=time_left)
+                            # Or should we do `if time_left > 0.001`?
+                            uid, x = q_in.get(timeout=max(0.001, time_left))
                         else:
                             uid, x = q_in.get_nowait()
                     except queue.Empty:
@@ -275,6 +275,11 @@ class MPServer(metaclass=ABCMeta):
     # with some enhancements related to data sharing between
     # processes.
 
+    SLEEP_ENQUEUE = 0.00015
+    SLEEP_DEQUEUE = 0.00011
+    TIMEOUT_ENQUEUE = 1
+    TIMEOUT_TOTAL = 10
+
     def __init__(self, *,
                  max_queue_size: int = None,
                  cpus: Sequence[int] = None,
@@ -286,12 +291,15 @@ class MPServer(metaclass=ABCMeta):
         self.max_queue_size = max_queue_size or 1024
         self._q_in_lock: List[synchronize.Lock] = []
         self._q_err: mp.Queue = self.MP_CLASS.Queue(self.max_queue_size)
+
         self._t_gather_results: threading.Thread = None  # type: ignore
+
         self._servlets: List[mp.Process] = []
         self._uid_to_futures = {}
 
         self.started = 0
         if cpus:
+            # Pin this coordinating thread to the specified CPUs.
             psutil.Process().cpu_affinity(cpus=cpus)
 
     def __del__(self):
@@ -323,7 +331,7 @@ class MPServer(metaclass=ABCMeta):
             k += 1
             logger.info(f"servlet processes ready: {k}/{n}")
 
-        self._t_gather_results = threading.Thread(target=self.gather_results)
+        self._t_gather_results = threading.Thread(target=self._gather_results)
         self._t_gather_results.start()
 
     def stop(self):
@@ -335,6 +343,7 @@ class MPServer(metaclass=ABCMeta):
 
         self._t_gather_results.join()
         self._t_gather_results = None
+
         for m in self._servlets:
             # if m.is_alive():
             m.terminate()
@@ -343,17 +352,6 @@ class MPServer(metaclass=ABCMeta):
         # Reset CPU affinity.
         psutil.Process().cpu_affinity(cpus=[])
 
-    def gather_results(self):
-        # This is not a "public" method in that end-user
-        # should not call this method. But this is an important
-        # top-level method.
-        while True:
-            if not self.started:
-                return
-            self._gather_output()
-            self._gather_error()
-            time.sleep(0.0013)
-
     async def async_call(self,
                          x,
                          *,
@@ -363,20 +361,13 @@ class MPServer(metaclass=ABCMeta):
         enqueue_timeout, total_timeout = self._resolve_timeout(
             enqueue_timeout=enqueue_timeout, total_timeout=total_timeout,
         )
-
-        loop = asyncio.get_running_loop()
-        fut = loop.create_future()
-        uid = self._enqueue_future(x, fut)
-
-        time0 = loop.time()
-
-        await self._async_call_enqueue(
-            x=x, uid=uid, fut=fut, qs=self._input_queues(),
-            t0=time0, enqueue_timeout=enqueue_timeout, loop=loop)
-
+        time0 = time.perf_counter()
+        uid, fut = await self._async_call_enqueue(
+            x, qs=self._input_queues(),
+            t0=time0, enqueue_timeout=enqueue_timeout)
         return await self._async_call_wait_for_result(
             uid=uid, fut=fut,
-            t0=time0, total_timeout=total_timeout, loop=loop)
+            t0=time0, total_timeout=total_timeout)
 
     def call(self,
              x,
@@ -396,20 +387,15 @@ class MPServer(metaclass=ABCMeta):
             enqueue_timeout=enqueue_timeout, total_timeout=total_timeout,
         )
 
-        fut = concurrent.futures.Future()
-        uid = self._enqueue_future(x, fut)
-
         time0 = time.perf_counter()
-
-        self._call_enqueue(x=x, uid=uid, fut=fut, qs=self._input_queues(),
-                           t0=time0, enqueue_timeout=enqueue_timeout)
-
+        uid, fut = self._call_enqueue(
+            x, qs=self._input_queues(),
+            t0=time0, enqueue_timeout=enqueue_timeout)
         return self._call_wait_for_result(
             uid=uid, fut=fut, t0=time0, total_timeout=total_timeout)
 
     def async_stream(self, data_stream, *,
                      return_exceptions: bool = False,
-                     output_buffer_size: int = 1024,
                      return_x: bool = False,
                      ):
         # What this method does can be achieved by a Streamer
@@ -417,48 +403,43 @@ class MPServer(metaclass=ABCMeta):
         # However, this method is expected to achieve optimal
         # performance, whereas the efficiency achieved by
         # "transfomer" depends on the "workers" parameter.
-        async def _enqueue(input_stream, future_stream, return_x):
-            loop = asyncio.get_running_loop()
-            q_ins = self._input_queues()
-            async for x in input_stream:
-                fut = loop.create_future()
-                uid = self._enqueue_future(x, fut)
 
-                # If one input queue is full, then the corresponding
-                # worker is slow. It does not help to enqueue this data
-                # into faster workers sooner. Hence we just enqueue
-                # for the workers one by one.
-                for q in q_ins:
-                    while True:
-                        if not self.started:
-                            return
-                        try:
-                            q.put_nowait((uid, x))
-                            break
-                        except queue.Full:
-                            await asyncio.sleep(0.0013)
+        enqueue_timeout, total_timeout = self._resolve_timeout()
 
-                if return_x:
-                    await future_stream.put((x, fut))
-                else:
-                    await future_stream.put(fut)
+        async def _enqueue(x, return_x):
+            time0 = time.perf_counter()
+            uid, fut = await self._async_call_enqueue(
+                x, qs=self._input_queues(),
+                t0=time0, enqueue_timeout=enqueue_timeout)
+            if return_x:
+                return x, fut, uid
+            else:
+                return fut, uid
 
-        if not isinstance(data_stream, async_streamer.Stream):
-            data_stream = async_streamer.Stream(data_stream)
-        q_fut = async_streamer.IterQueue(
-            output_buffer_size, data_stream.in_stream)
-        _ = async_streamer.streamer_task(
-            data_stream.in_stream, q_fut, _enqueue, return_x)
+        async def _dequeue(x, return_x):
+            if return_x:
+                x, fut, uid = x
+            else:
+                fut, uid = x
+            time0 = time.perf_counter()
+            z = await self._async_call_wait_for_result(
+                uid=uid, fut=fut,
+                t0=time0, total_timeout=total_timeout)
+            if return_x:
+                return x, z
+            return z
 
-        q_out = async_streamer.IterQueue(q_fut.maxsize, q_fut)
-        _ = async_streamer.streamer_task(
-            q_fut, q_out, self._async_stream_dequeue, return_exceptions, return_x)
+        if not isinstance(data_stream, streamer.AsyncStream):
+            data_stream = streamer.AsyncStream(data_stream)
 
-        return async_streamer.Stream(q_out)
+        return data_stream.transform(
+            _enqueue, return_x=return_x,
+            return_exceptions=return_exceptions).transform(
+                _dequeue, return_x=return_x,
+                return_exceptions=return_exceptions)
 
     def stream(self, data_stream, *,
                return_exceptions: bool = False,
-               output_buffer_size: int = 1024,
                return_x: bool = False,
                ):
         # What this method does can be achieved by a Streamer
@@ -466,32 +447,42 @@ class MPServer(metaclass=ABCMeta):
         # However, this method is expected to achieve optimal
         # performance, whereas the efficiency achieved by
         # "transfomer" depends on the "workers" parameter.
-        def _enqueue(input_stream, future_stream, return_x):
-            q_ins = self._input_queues()
-            for x in input_stream:
-                fut = concurrent.futures.Future()
-                uid = self._enqueue_future(x, fut)
-                for q in q_ins:
-                    if not self.started:
-                        return
-                    q.put((uid, x))
-                if return_x:
-                    future_stream.put((x, fut))
-                else:
-                    future_stream.put(fut)
+
+        enqueue_timeout, total_timeout = self._resolve_timeout()
+
+        def _enqueue(x, return_x):
+            fut = concurrent.futures.Future()
+            time0 = time.perf_counter()
+            uid, fut = self._call_enqueue(
+                x, qs=self._input_queues(),
+                t0=time0, enqueue_timeout=enqueue_timeout)
+            if return_x:
+                return x, uid, fut
+            else:
+                return uid, fut
+
+        def _dequeue(x, return_x):
+            if return_x:
+                x, uid, fut = x
+            else:
+                uid, fut = x
+            time0 = time.perf_counter()
+            z = self._call_wait_for_result(
+                uid=uid, fut=fut,
+                t0=time0, total_timeout=total_timeout)
+            z = fut.result()
+            if return_x:
+                return x, z
+            return z
 
         if not isinstance(data_stream, streamer.Stream):
-            data_stream = streamer.Stream(
-                data_stream, maxsize=output_buffer_size)
-        q_fut = streamer.IterQueue(output_buffer_size, data_stream.in_stream)
-        _ = streamer.streamer_thread(
-            data_stream.in_stream, q_fut, _enqueue, return_x)
+            data_stream = streamer.Stream(data_stream)
 
-        q_out = streamer.IterQueue(q_fut.maxsize, q_fut)
-        _ = streamer.streamer_thread(
-            q_fut, q_out, self._stream_dequeue, return_exceptions, return_x)
-
-        return streamer.Stream(q_out)
+        return data_stream.transform(
+            _enqueue, workers=1, return_x=return_x,
+            return_exceptions=return_exceptions).transform(
+                _dequeue, workers=1, return_x=return_x,
+                return_exceptions=return_exceptions)
 
     def _add_servlet(self,
                      servlet: Type[Servlet],
@@ -576,11 +567,11 @@ class MPServer(metaclass=ABCMeta):
                          total_timeout: float = None):
 
         if enqueue_timeout is None:
-            enqueue_timeout = 1
+            enqueue_timeout = self.TIMEOUT_ENQUEUE
         elif enqueue_timeout < 0:
             enqueue_timeout = 0
         if total_timeout is None:
-            total_timeout = max(10, enqueue_timeout * 10)
+            total_timeout = max(self.TIMEOUT_TOTAL, enqueue_timeout * 10)
         else:
             assert total_timeout > 0, "total_timeout must be > 0"
         if enqueue_timeout > total_timeout:
@@ -592,19 +583,31 @@ class MPServer(metaclass=ABCMeta):
         raise NotImplementedError
 
     def _enqueue_future(self, x, fut):
+        z = self._init_future_struct(x, fut)
         while True:
             uid = str(uuid4())
-            if uid in self._uid_to_futures:
-                continue
-            self._uid_to_futures[uid] = self._init_future_struct(x, fut)
-            return uid
+            if self._uid_to_futures.setdefault(uid, z) is z:
+                return uid
 
     @abstractmethod
     def _input_queues(self):
         raise NotImplementedError
 
+    def _gather_results(self):
+        # This is not a "public" method in that end-user
+        # should not call this method. But this is an important
+        # top-level method.
+        while True:
+            if not self.started:
+                return
+            self._gather_output()
+            self._gather_error()
+            time.sleep(self.SLEEP_DEQUEUE)
+
     @abstractmethod
     def _gather_output(self):
+        # Subclass implementation of this function is responsible
+        # for removing the finished entry from `self._uid_to_futures`.
         raise NotImplementedError
 
     def _gather_error(self):
@@ -628,41 +631,11 @@ class MPServer(metaclass=ABCMeta):
                     raise
             # No sleep. Get results out of the queue as quickly as possible.
 
-    async def _async_call_enqueue(self, *, x, uid, fut, qs, t0, enqueue_timeout, loop):
-        t1 = t0 + enqueue_timeout
-        for iq, q in enumerate(qs):
-            while True:
-                if not self.started:
-                    return
-                try:
-                    q.put_nowait((uid, x))
-                    break
-                except queue.Full:
-                    timenow = loop.time()
-                    if timenow < t1:
-                        await asyncio.sleep(0.00089)
-                    else:
-                        fut.cancel()
-                        del self._uid_to_futures[uid]
-                        raise EnqueueTimeout(
-                            f'waited {timenow - t0} seconds; timeout at queue {iq}')
+    async def _async_call_enqueue(self, x, *, qs, t0, enqueue_timeout):
+        loop = asyncio.get_running_loop()
+        fut = loop.create_future()
+        uid = self._enqueue_future(x, fut)
 
-    async def _async_call_wait_for_result(self, *, uid, fut, t0, total_timeout, loop):
-        try:
-            await asyncio.wait_for(fut, timeout=t0 + total_timeout - loop.time())
-            # This could raise MPError.
-        except asyncio.TimeoutError:
-            # `fut` is now cancelled.
-            if uid in self._uid_to_futures:
-                # `uid` could have been deleted by
-                # `gather_results` during very subtle
-                # timing coincidence.
-                del self._uid_to_futures[uid]
-            raise TotalTimeout(f'waited {loop.time() - t0} seconds')
-        else:
-            return fut.result()
-
-    def _call_enqueue(self, *, x, uid, fut, qs, t0, enqueue_timeout):
         t1 = t0 + enqueue_timeout
         for iq, q in enumerate(qs):
             while True:
@@ -674,77 +647,68 @@ class MPServer(metaclass=ABCMeta):
                 except queue.Full:
                     timenow = time.perf_counter()
                     if timenow < t1:
-                        time.sleep(0.00089)
+                        await asyncio.sleep(self.SLEEP_ENQUEUE)
                     else:
                         fut.cancel()
                         del self._uid_to_futures[uid]
                         raise EnqueueTimeout(
                             f'waited {timenow - t0} seconds; timeout at queue {iq}')
+        return uid, fut
 
-    def _call_wait_for_result(self, *, uid, fut, t0, total_timeout):
-        try:
-            z = fut.result(timeout=t0 + total_timeout - time.perf_counter())
-            # This could raise MPError.
-        except (asyncio.TimeoutError, concurrent.futures.TimeoutError):
-            # `fut` is now cancelled.
-            if uid in self._uid_to_futures:
+    async def _async_call_wait_for_result(self, *, uid, fut, t0, total_timeout):
+        t1 = t0 + total_timeout
+        while not fut.done():
+            if time.perf_counter() > t1:
+                fut.cancel()
+                self._uid_to_futures.pop(uid, None)
                 # `uid` could have been deleted by
                 # `gather_results` during very subtle
                 # timing coincidence.
-                del self._uid_to_futures[uid]
-            raise TotalTimeout(f'waited {time.perf_counter() - t0} seconds')
-        else:
-            return z
+                raise TotalTimeout(
+                    f'waited {time.perf_counter() - t0} seconds')
+            await asyncio.sleep(self.SLEEP_DEQUEUE)
+        # await asyncio.wait_for(fut, timeout=t0 + total_timeout - time.perf_counter())
+        # NOTE: `asyncio.wait_for` seems to be blocking for the
+        # `timeout` even after result is available.
+        return fut.result()
+        # This could raise MPError.
 
-    async def _async_stream_dequeue(self, q_fut, q_out, return_exceptions, return_x):
-        if return_x:
-            async for x, fut in q_fut:
-                try:
-                    z = await fut
-                    await q_out.put((x, z))
-                except Exception as e:
-                    if return_exceptions:
-                        await q_out.put((x, e))
-                    else:
-                        raise e
-        else:
-            async for fut in q_fut:
-                try:
-                    z = await fut
-                    await q_out.put(z)
-                except Exception as e:
-                    if return_exceptions:
-                        await q_out.put(e)
-                    else:
-                        if isinstance(e, MPError):
-                            logger.error(e.trace_back)
-                        raise e
+    def _call_enqueue(self, x, *, qs, t0, enqueue_timeout):
+        fut = concurrent.futures.Future()
+        uid = self._enqueue_future(x, fut)
 
-    def _stream_dequeue(self, q_fut, q_out, return_exceptions, return_x):
-        if return_x:
-            for x, fut in q_fut:
+        t1 = t0 + enqueue_timeout
+        for iq, q in enumerate(qs):
+            while True:
+                if not self.started:
+                    return
                 try:
-                    z = fut.result()
-                    q_out.put((x, z))
-                except Exception as e:
-                    if return_exceptions:
-                        q_out.put((x, e))
+                    q.put_nowait((uid, x))
+                    break
+                except queue.Full:
+                    timenow = time.perf_counter()
+                    if timenow < t1:
+                        time.sleep(self.SLEEP_ENQUEUE)
                     else:
-                        if isinstance(e, MPError):
-                            logger.error(e.trace_back)
-                        raise e
-        else:
-            for fut in q_fut:
-                try:
-                    z = fut.result()
-                    q_out.put(z)
-                except Exception as e:
-                    if return_exceptions:
-                        q_out.put(e)
-                    else:
-                        if isinstance(e, MPError):
-                            logger.error(e.trace_back)
-                        raise e
+                        fut.cancel()
+                        del self._uid_to_futures[uid]
+                        raise EnqueueTimeout(
+                            f'waited {timenow - t0} seconds; timeout at queue {iq}')
+        return uid, fut
+
+    def _call_wait_for_result(self, *, uid, fut, t0, total_timeout):
+        t1 = t0 + total_timeout
+        while not fut.done():
+            if time.perf_counter() > t1:
+                fut.cancel()
+                self._uid_to_futures.pop(uid, None)
+                # `uid` could have been deleted by
+                # `gather_results` during very subtle
+                # timing coincidence.
+                raise TotalTimeout(
+                    f'waited {time.perf_counter() - t0} seconds')
+            time.sleep(self.SLEEP_DEQUEUE)
+        return fut.result()
 
 
 class SequentialServer(MPServer):
